﻿//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.IO;
using System.Xml;


namespace MindFusion.Scheduling.Wpf.Samples.CS.Clipboard
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			_menuCopy = new MenuItem();
			_menuCopy.Header = "Copy";
			_menuCopy.Click += new RoutedEventHandler(_menuCopy_Click);

			_menuPaste = new MenuItem();
			_menuPaste.Header = "Paste";
			_menuPaste.Click += new RoutedEventHandler(_menuPaste_Click);

			contextMenu = new ContextMenu();
			contextMenu.Items.Add(_menuCopy);
			contextMenu.Items.Add(_menuPaste);

			_schedule.ContextMenu = contextMenu;
		}

		void _menuPaste_Click(object sender, RoutedEventArgs e)
		{
			// Check data availability
			IDataObject data = System.Windows.Clipboard.GetDataObject();
			if (!data.GetDataPresent(typeof(ItemData)))
				return;

			string[] formats = data.GetFormats();
			object oo = data.GetData(formats[0]);
			ItemData itemData = data.GetData(typeof(ItemData)) as ItemData;

			// Create the item
			Item newItem = null;
			if (itemData.Type == "MindFusion.Scheduling.Appointment")
				newItem = new Appointment();

			if (newItem == null)
				return;

			// Deserialize the item
			MemoryStream stream = new MemoryStream(itemData.Data);

			XmlDocument doc = new XmlDocument();
			doc.Load(stream);
			stream.Close();

			XmlSerializationContext context = new XmlSerializationContext(_schedule.Schedule, doc);

			newItem.LoadFrom(doc.SelectSingleNode("item") as XmlElement, context);

			// Set the appropriate start and end time for the
			// newly created item
			TimeSpan length = newItem.EndTime - newItem.StartTime;
			newItem.StartTime = _clickedDate;
			newItem.EndTime = _clickedDate + length;

			// Ensure the item has unique id
			newItem.Id = Guid.NewGuid().ToString();

			// Add the item to the schedule
			_schedule.Schedule.Items.Add(newItem);
		}

		void _menuCopy_Click(object sender, RoutedEventArgs e)
		{
			if (_clickedItem == null)
				return;

			// Copy the clicked item
			XmlDocument doc = new XmlDocument();
			XmlSerializationContext context = new XmlSerializationContext(_schedule.Schedule, doc);

			XmlElement itemElement = doc.CreateElement("item");
			doc.AppendChild(itemElement);

			_clickedItem.SaveTo(itemElement, context);

			MemoryStream stream = new MemoryStream();
			doc.Save(stream);

			DataObject data = new DataObject(
				new ItemData(_clickedItem.GetType().FullName, stream.GetBuffer()));
			System.Windows.Clipboard.SetDataObject(data, true);
			stream.Close();
		}

		void _schedule_DateClick(object sender, ResourceDateEventArgs e)
		{
			if (e.RightButton == MouseButtonState.Pressed)
			{
				// Select the right-clicked date
				_schedule.Selection.Set(e.Date, e.Date + _schedule.TimetableSettings.CellTime, null, null, null, null);

				// Remember the date
				_clickedDate = e.Date;

				// Update the menu
				IDataObject data = System.Windows.Clipboard.GetDataObject();
				_menuPaste.IsEnabled = data.GetDataPresent(typeof(ItemData));
			}
		}

		void _schedule_ItemClick(object sender, ItemMouseEventArgs e)
		{
			if (e.RightButton == MouseButtonState.Pressed)
			{
				// Select the right-clicked item
				_schedule.ItemSelection.Clear();
				_schedule.ItemSelection.Add(e.Item);

				// Remember the item
				_clickedItem = e.Item;

				// Check data availability
				IDataObject data = System.Windows.Clipboard.GetDataObject();
				_menuPaste.IsEnabled = data.GetDataPresent(typeof(ItemData));
			}
		}


		//private Calendar _schedule;
		private ContextMenu contextMenu;
		private MenuItem _menuCopy;
		private MenuItem _menuPaste;
		private Item _clickedItem;
		private DateTime _clickedDate;
	}

	[Serializable]
	public class ItemData
	{
		public ItemData(string type, byte[] data)
		{
			_type = type;
			_data = data;
		}


		public string Type
		{
			get { return _type; }
		}

		public byte[] Data
		{
			get { return _data; }
		}


		private string _type;
		private byte[] _data;
	}
}
